@@ -49,7 +49,7 @@ def colormap(graph, name, inputs):
     image: :class:`imagecat.data.Image`
         A copy of the input image with some layers mapped.
     """
-    image = imagecat.operator.util.require_image(name, inputs, "image", index=0)
+    image = imagecat.operator.util.require_image(name, inputs, "image")
     layers = imagecat.operator.util.optional_input(name, inputs, "layers", type=str, default="*")
     mapping = imagecat.operator.util.optional_input(name, inputs, "mapping", default=None)
 
@@ -98,37 +98,31 @@ def composite(graph, name, inputs):
     image: :class:`imagecat.data.Image`
         New image with a single solid-color layer.
     """
-    bglayer = imagecat.operator.util.optional_input(name, inputs, "bglayer", index=0, type=str, default="C")
-    fglayer = imagecat.operator.util.optional_input(name, inputs, "fglayer", index=0, type=str, default="C")
-    layer = imagecat.operator.util.optional_input(name, inputs, "layer", index=0, type=str, default=bglayer)
-    masklayer = imagecat.operator.util.optional_input(name, inputs, "masklayer", index=0, type=str, default="A")
-    order = imagecat.operator.util.optional_input(name, inputs, "order", index=0, type=int, default=3)
-    orientation = imagecat.operator.util.optional_input(name, inputs, "orientation", index=0, type=float, default=0)
-    pivot = imagecat.operator.util.optional_input(name, inputs, "pivot", index=0, default=["0.5w", "0.5h"])
-    position = imagecat.operator.util.optional_input(name, inputs, "position", index=0, default=["0.5w", "0.5h"])
-    scale = imagecat.operator.util.optional_input(name, inputs, "scale", index=0, default=[1, 1])
+    bglayer = imagecat.operator.util.optional_input(name, inputs, "bglayer", type=str, default="C")
+    fglayer = imagecat.operator.util.optional_input(name, inputs, "fglayer", type=str, default="C")
+    layer = imagecat.operator.util.optional_input(name, inputs, "layer", type=str, default=bglayer)
+    masklayer = imagecat.operator.util.optional_input(name, inputs, "masklayer", type=str, default="A")
+    order = imagecat.operator.util.optional_input(name, inputs, "order", type=int, default=3)
+    orientation = imagecat.operator.util.optional_input(name, inputs, "orientation", type=float, default=0)
+    pivot = imagecat.operator.util.optional_input(name, inputs, "pivot", default=["0.5w", "0.5h"])
+    position = imagecat.operator.util.optional_input(name, inputs, "position", default=["0.5w", "0.5h"])
+    scale = imagecat.operator.util.optional_input(name, inputs, "scale", default=[1, 1])
 
-    background = imagecat.operator.util.require_layer(name, inputs, "background", index=0, layer=bglayer)
-    foreground = imagecat.operator.util.require_layer(name, inputs, "foreground", index=0, layer=fglayer)
-    mask = imagecat.operator.util.optional_layer(name, inputs, "mask", index=0, layer=masklayer, components=1)
-
-#    Mysteriously, this version is slightly *slower* than the below.
-#    data = numpy.dstack((foreground.data, mask.data))
-#    data = imagecat.operator.util.transform(data, background.data.shape, pivot=pivot, orientation=orientation, position=position)
-#    alpha = data[:,:,3:4]
-#    data = data[:,:,0:3]
-#    data = data * alpha + background.data * (1 - alpha)
+    background = imagecat.operator.util.require_layer(name, inputs, "background", layer=bglayer)
+    foreground = imagecat.operator.util.require_layer(name, inputs, "foreground", layer=fglayer)
+    mask = imagecat.operator.util.optional_layer(name, inputs, "mask", layer=masklayer, components=1)
 
     if mask is None:
         mask = numpy.ones((foreground.shape[0], foreground.shape[1], 1), dtype=numpy.float16)
     else:
         mask = mask.data
 
-    transformed_foreground = imagecat.operator.util.transform(foreground.data, background.data.shape, pivot=pivot, orientation=orientation, position=position, scale=scale, order=order)
-    transformed_mask = imagecat.operator.util.transform(mask, background.data.shape, pivot=pivot, orientation=orientation, position=position, scale=scale, order=order)
+    i1, i2, j1, j2, transformed_foreground = imagecat.operator.util.transform(foreground.data, background.data.shape, pivot=pivot, orientation=orientation, position=position, scale=scale, order=order)
+    i1, i2, j1, j2, transformed_mask = imagecat.operator.util.transform(mask, background.data.shape, pivot=pivot, orientation=orientation, position=position, scale=scale, order=order)
     alpha = transformed_mask
     one_minus_alpha = 1 - alpha
-    data = transformed_foreground * alpha + background.data * one_minus_alpha
+    data = background.data.copy()
+    data[i1:i2, j1:j2] = transformed_foreground * alpha + data[i1:i2, j1:j2] * one_minus_alpha
 
     output = imagecat.data.Image(layers={layer: imagecat.data.Layer(data=data, components=background.components, role=background.role)})
     imagecat.operator.util.log_result(log, name, "composite", output, bglayer=bglayer, fglayer=fglayer, masklayer=masklayer, layer=layer, order=order, orientation=orientation, pivot=pivot, position=position, scale=scale)
@@ -155,7 +149,7 @@ def delete(graph, name, inputs):
     image: :class:`imagecat.data.Image`
         A copy of the input image with some layers deleted.
     """
-    image = imagecat.operator.util.require_image(name, inputs, "image", index=0)
+    image = imagecat.operator.util.require_image(name, inputs, "image")
     layers = imagecat.operator.util.optional_input(name, inputs, "layers", type=str, default="*")
 
     remove = image.match_layer_names(layers)
@@ -225,7 +219,7 @@ def gaussian(graph, name, inputs):
     """
     import skimage.filters
 
-    image = imagecat.operator.util.require_image(name, inputs, "image", index=0)
+    image = imagecat.operator.util.require_image(name, inputs, "image")
     layers = imagecat.operator.util.optional_input(name, inputs, "layers", type=str, default="*")
     radius = imagecat.operator.util.optional_input(name, inputs, "radius", default=["5px", "5px"])
 
@@ -282,9 +276,9 @@ def merge(graph, name, inputs):
 
     This operator merges the layers from multiple images into a single image.
     The images must all have the same resolution.  Upstream inputs are merged
-    in order, sorted by input name and index.  Layter image layers with
-    duplicate names will overwrite earlier layers.  Callers can prevent this by
-    renaming layers upstream with the :func:`rename` operator.
+    in order, sorted by input name.  Later image layers with duplicate names
+    will overwrite earlier layers.  Callers can prevent this by renaming layers
+    upstream with the :func:`rename` operator.
 
     See Also
     --------
@@ -308,11 +302,10 @@ def merge(graph, name, inputs):
         New image containing the union of all input layers.
     """
     output = imagecat.data.Image()
-    for input in sorted(inputs.keys()):
-        for index in range(len(inputs[input])):
-            image = inputs[input][index]
-            if isinstance(image, imagecat.data.Image):
-                output.layers.update(image.layers)
+    for input, value in sorted(inputs.items()):
+        image = value()
+        if isinstance(image, imagecat.data.Image):
+            output.layers.update(image.layers)
     imagecat.operator.util.log_result(log, name, "merge", output)
     return output
 
@@ -338,9 +331,9 @@ def offset(graph, name, inputs):
     image: :class:`imagecat.data.Image`
         A copy of the input image with some layers offset.
     """
-    image = imagecat.operator.util.require_image(name, inputs, "image", index=0)
-    layers = imagecat.operator.util.optional_input(name, inputs, "layers", index=0, type=str, default="*")
-    offset = imagecat.operator.util.optional_input(name, inputs, "offset", index=0, default=["0.5w", "0.5h"])
+    image = imagecat.operator.util.require_image(name, inputs, "image")
+    layers = imagecat.operator.util.optional_input(name, inputs, "layers", type=str, default="*")
+    offset = imagecat.operator.util.optional_input(name, inputs, "offset", default=["0.5w", "0.5h"])
 
     output = imagecat.data.Image()
     for layer_name in image.match_layer_names(layers):
@@ -403,7 +396,7 @@ def rgb2gray(graph, name, inputs):
     image: :class:`imagecat.data.Image`
         A copy of the input image with some layers converted to grayscale.
     """
-    image = imagecat.operator.util.require_image(name, inputs, "image", index=0)
+    image = imagecat.operator.util.require_image(name, inputs, "image")
     layers = imagecat.operator.util.optional_input(name, inputs, "layers", type=str, default="*")
     weights = imagecat.operator.util.optional_input(name, inputs, "weights", type=imagecat.operator.util.array(shape=(3,)), default=[0.2125, 0.7154, 0.0721])
 
@@ -437,7 +430,7 @@ def save(graph, name, inputs):
         :["path"][0]: :class:`str`, required. Filesystem path of the file to be saved.
         :["layers"][0]: :class:`str`, optional. Pattern matching the layers to be saved.  Default: '*', which saves all layers.
     """
-    image = imagecat.operator.util.require_image(name, inputs, "image", index=0)
+    image = imagecat.operator.util.require_image(name, inputs, "image")
     path = imagecat.operator.util.require_input(name, inputs, "path", type=str)
     layers = imagecat.operator.util.optional_input(name, inputs, "layers", type=str, default="*")
 
@@ -471,7 +464,7 @@ def resize(graph, name, inputs):
     """
     import skimage.transform
 
-    image = imagecat.operator.util.require_image(name, inputs, "image", index=0)
+    image = imagecat.operator.util.require_image(name, inputs, "image")
     order = imagecat.operator.util.optional_input(name, inputs, "order", type=int, default=3)
     res = imagecat.operator.util.optional_input(name, inputs, "res", default=("1w", "1h"))
 
@@ -565,7 +558,9 @@ def uniform(graph, name, inputs):
         New image with one layer containing uniform noise.
     """
     components = imagecat.operator.util.optional_input(name, inputs, "components", default=None)
+    high = imagecat.operator.util.optional_input(name, inputs, "high", type=float, default=1)
     layer = imagecat.operator.util.optional_input(name, inputs, "layer", type=str, default="A")
+    low = imagecat.operator.util.optional_input(name, inputs, "low", type=float, default=0)
     role = imagecat.operator.util.optional_input(name, inputs, "role", type=imagecat.data.Role, default=imagecat.data.Role.NONE)
     seed = imagecat.operator.util.optional_input(name, inputs, "seed", type=int, default=1234)
     res = imagecat.operator.util.optional_input(name, inputs, "res", type=imagecat.operator.util.array(shape=(2,), dtype=int), default=[256, 256])
@@ -574,9 +569,9 @@ def uniform(graph, name, inputs):
         components = [""]
 
     generator = numpy.random.default_rng(seed=seed)
-    data = generator.uniform(size=(res[1], res[0], len(components))).astype(numpy.float16)
+    data = generator.uniform(low=low, high=high, size=(res[1], res[0], len(components))).astype(numpy.float16)
     output = imagecat.data.Image(layers={layer: imagecat.data.Layer(data=data, components=components, role=role)})
-    imagecat.operator.util.log_result(log, name, "uniform", output, components=components, layer=layer, role=role, seed=seed, res=res)
+    imagecat.operator.util.log_result(log, name, "uniform", output, components=components, low=low, high=high, layer=layer, role=role, seed=seed, res=res)
     return output
 
 
